#####################################################################
# This code is written and copyrighted by Clif Flynt, 2000
#
# It is licenced for public use with these terms:
#
#  This licence agreement must remain intact with the code.
# 
#  You may use this code freely for non-commercial purposes.
#
#  You may examine and modify the code to your heart's content.
#
#  You may not claim to have authored the code (aside from modifications
#  you may have made.)
#
#  You may not distribute the code without this license agreement
#
#  You may not sell this code or distribute it with a commercial product
#      without arrangements with Noumena Corporation.
#
#  Maintenance, extension, modification, etc can be performed by:
#
#   Noumena Corporation
#   9300 Fleming Rd.
#   Dexter, MI  48130
#
#   Contact: clif@noucorp.com
#
#####################################################################
set rev(Client_base.tcl) {$Header: /home/clif/CVS_ROOT/ClientServer/Client_base.tcl,v 1.5 2000/09/06 20:52:30 clif Exp $}

#
# lappend auto_path "Where tools are"
lappend auto_path ../Tools .

package require cmd_tools

# Default port and address - can be reassigned on command line
# as -port 99999 -ipAddr 999.999.999.999
set Client(port) 25327
set Client(ipAddr) 127.0.0.1

# The time to wait after opening a connection to initiate a conversation.

set Client(time) 1000

# If keepalive is set, the client will attempt to reopen a connection
#  to the server if the current connection dies.

set Client(keepalive) 1

parseArgs Client

################################################################
# proc bgerror {message}--
#    Handles background errors.
# Arguments
#   message	An error message
# 
# Results
#   Displays this error message, and the stack.  Continues running.
# 
proc bgerror {message} {
   global errorInfo
   
   puts "ERROR: $message"
   puts "$errorInfo"
   
}

################################################################
#  proc ClientPush {channel {string {}}} --
#    Sends an initial message for client/server relationships where
#   the client needs to initiate things.
# Arguments
#   channel 	The channel that was just opened
#   string	An optional string to transmit.
#               If the string is not presented in the invocation,
#               it should be defined in the ClientPush body.
# 
# Results
#   A message is sent to the server, and a response is probably 
#   caught in readline.
#   
 proc ClientPush {channel {string {}}} {
  global Client
  puts $channel $string
  flush $channel
 
 }


################################################################
# proc readLine {channel}--
#    
# Read a line of data, Add processing to this proc.
#
# Arguments
#   The channel to read from
# 
# Results
#   Reads input, and may process it.
#   If channel closes and keepalive is set this will schedule a new ClientOpen
#   for 10 seconds (by default)

proc readLine {channel} {
   global interpName cmd cmdCount errorInfo
   global Client
   
   set len [gets $channel line]
      
   # if we read 0 chars, check for EOF.  Close the
   #   channel and delete that client entry if we've
   #   hit the end of the road.
    
     if {($len <= 0) & [eof $channel]} {
         close $channel

         if {$Client(keepalive)} {
             after 6000 "ClientOpen $Client(ipAddr) $Client(port)"
         }

         return
     } else {
        if {[string match $line "quit"]} {
	    set Client(allDone) 1
	}
	append Client(input.$channel) " " $line

	if {[info complete $Client(input.$channel)]} {
	  #    The processing is done assynchronously (using the after)
	  #  to allow the client to stack requests if necessary.
	  #  Putting the data in the stack means that we can even stack
	  #  requests from a single client if necessary.

	  after $Client(time) [list processData $channel $Client(input.$channel)]
	  set Client(input.$channel) ""
	} 
     }
}

################################################################
# proc processData {channel input}--
#    Process the data received in a readLine command.
# Arguments
#   channel	The channel which received the data
#   input	The data received.
# 
# Results
#   Undefined.  What do you want the results to be?
# 
proc processData {channel input} {
   global Client

        #
	# Add special purpose processing code here.
	#  
	#  DANGER--DANGER--DANGER
	#
	#  This example code will evaluate a Tcl command sent
	#  by the server.  
	#  
	#  This is probably *NOT* what you want to do.  
	#  At the least, consider using a child interpreter!

    puts "Got: $input"
    eval $input
    
    puts $channel "puts {client is OK}"
    puts $channel "shutdown server"
}

################################################################
# proc goodaddress {addr}--
#    Check that the connection IP address is valid.
#    Returns a boolean suitable for use in an if {[goodaddress xx]}  statement.
# Arguments
#   addr:	The address to check
# 
# Results
#   Returns a 1 or 0 for OK or bad address.
# 
proc goodaddress {addr} {

    if {([string first 192.168.9 $addr] != 0) && 
        ([string first 127.0.0.1 $addr] != 0)} {
        return 0
    }
    return 1
}

################################################################
# proc ClientOpen {addr port}--
#    Opens a connection to the server.
# Arguments
#   addr:	The IP Address of the server.
#   port:	The port to connect to.
# Results
#   
# 
proc ClientOpen {addr port} {
   global Client
   
   # Confirm that we are contacting a valid server address
   # If this client/server pair should only be used on a local network, 
   # goodaddress is the place to confirm that it will only contact
   # valid addresses.
   # Think of it as an application specific Access Control List.
 
   if {![goodaddress $addr]} {
       return
   }
    
   set fail [catch {socket $addr $port} channel ]

   # If we can't get a connection now, try again later.

   if {$fail} {
       if {$Client(keepalive)} {
       after 6000 "ClientOpen $addr $port"  
       return;
       } else {
           global errorInfo
           error $errorInfo
       }
   }
   
   # Save the channel associated with this address and port
   # just in case we want it again later.
   #  (The client might open several server connections.)

   set Client($addr.$port) $channel
   
   # Initialize the input buffer, so we can check for a complete
   #  command easily.

   set Client(input.$channel) ""

   # Set the fileevent to grab input from the channel.
    
   fileevent $channel readable "readLine $channel"

   # This sample  buffers by line.
   # Your mileage may vary.

   fconfigure $channel -buffering line

   # If we need to do something to kick off the action,
   # wait a specified length of time, and kick it off.
 
   if {![string match [info procs ClientPush] ""]} {
      after $Client(time) "ClientPush $channel"
   }
}

set Client(allDone) 0

set start [clock clicks]
ClientOpen $Client(ipAddr) $Client(port)
puts "back [expr [clock clicks] - $start]"

catch {vwait Client(allDone)}

