#####################################################################
# This code is written and copyrighted by Clif Flynt, 2000
#
# It is licenced for public use with these terms:
#
#  This licence agreement must remain intact with the code.
# 
#  You may use this code freely for non-commercial purposes.
#
#  You may examine and modify the code to your heart's content.
#
#  You may not claim to have authored the code (aside from modifications
#  you may have made.)
#
#  You may not distribute the code without this license agreement
#
#  You may not sell this code or distribute it with a commercial product
#      without arrangements with Noumena Corporation.
#
#  Maintenance, extension, modification, etc can be performed by:
#
#   Noumena Corporation
#   9300 Fleming Rd.
#   Dexter, MI  48130
#
#   Contact: clif@noucorp.com
#
#####################################################################
set rev(Server_base.tcl) {$Header: /home/clif/CVS_ROOT/ClientServer/Server_base.tcl,v 1.3 2000/09/06 20:52:30 clif Exp $}
#
# lappend auto_path "Where tools are
lappend auto_path ../Tools .

package require cmd_tools

set Server(port) 25327
set Server(time) 1000

parseArgs Server

################################################################
# proc bgerror {message}--
#    Handles background errors.
# Arguments
#   message	An error message
# 
# Results
#   Displays this error message, and the stack.  Continues running.
# 
proc bgerror {message} {
   global errorInfo
   
   puts "ERROR: $message"
   puts "$errorInfo"
   
}

################################################################
#  proc ServerPush {channel {string {}}} --
#    Sends an initial message for client/server relationships where
#   the server needs to initiate things.
# Arguments
#   channel 	The channel that was just opened
#   string	An optional string to transmit.
#               If the string is not presented in the invocation,
#               it should be defined in the ServerPush body.
# 
# Results
#   A message is sent to the server, and a response is probably 
#   caught in readline.
#   
 proc ServerPush {channel {string {}} {opt {-nonewline} } } {
  global Server
  
  eval puts $opt $channel [list $string]

  flush $channel
 
 }

# Read a line of data

proc readLine {channel} {
   global interpName cmd cmdCount errorInfo
   global Server
   
   set len [gets $channel line]
      
   # if we read 0 chars, check for EOF.  Close the
   #   channel and delete that client entry if we've
   #   hit the end of the road.

     if {($len <= 0) & [eof $channel]} {
         close $channel
         return
     } else {
        if {[string match $line "quit"]} {
            close $channel
            return
            }
	    
	# This can be useful while testing, but probably won't
	# be a clever idea in a production server.

        if {[string match $line "shutdown server"]} {
            set Server(allDone) 1
            return
        }


	append Server(input.$channel) " " $line

	# When we have a complete line, process the line.
	#
	# NOTE: You may need other tests here for command completion.
	# This test only checks that quotes and braces match and are
	#  in complete pairs.

	if {[info complete $Server(input.$channel)]} {
	  #    The processing is done assynchronously (using the after)
	  #  to allow the server to stack requests if necessary.
	  #  Putting the data in the stack means that we can even stack
	  #  requests from a single client if necessary.

	  after $Server(time) [list processData $channel $Server(input.$channel)]
	  set Server(input.$channel) ""
	} 
     }
}

################################################################
# proc processData {channel input}--
#    Process the data received in a readLine command.
# Arguments
#   channel	The channel which received the data
#   input	The data received.
# 
# Results
#   Undefined.  What do you want the results to be?
# 
proc processData {channel input} {
   global Server

        #
	# Add special purpose processing code here.
	#  
	#  DANGER--DANGER--DANGER
	#
	#  This example code will evaluate a Tcl command sent
	#  by the server.  
	#  
	#  This is probably *NOT* what you want to do.  
	#  At the least, consider using a child interpreter!

    eval $input

}

################################################################
# proc goodaddress {addr}--
#    Check that the connection IP address is valid.
#    Returns a boolean suitable for use in an if {[goodaddress xx]}  statement.
# Arguments
#   addr:	The address to check
# 
# Results
#   Returns a 1 or 0 for OK or bad address.
# 
proc goodaddress {addr} {

    if {([string first 192.168.9 $addr] != 0) && 
        ([string first 127.0.0.1 $addr] != 0)} {
        return 0
    }
    return 1
}


################################################################
# proc serverOpen {channel addr port} --
#    Invoked when the socket is opened.
#    Handles the setup for a client socket.
# Arguments
#   channel   	The channel assigned by the Tcl socket command
#   addr	The IP address of the client.
#   port	The port assigned to this connection.
# 
# Results
#   Sets up a fileevent to handle input.
#   Configures the socket to be line buffered.
#   Saves the ip address and port in case it's needed again.

proc serverOpen {channel addr port} {
   global Server
puts "OPENING CHANNEL: $channel"
   # Set up fileevent to be called when input is available
   
   if {![goodaddress $addr]} {
       close $channel
       return
   }
       
   fileevent $channel readable "readLine $channel"
   fconfigure $channel -buffering line
   set Server(channel.$channel) [list $addr $port]

   # Possibly provide a kick start.

   if {![string match [info procs ServerPush] ""]} {

# A test push - to cause the client to display "Server: OK"
      after $Server(time) ServerPush  [list $channel {puts "Server: OK"} {}]
#      after $Server(time) ServerPush  $channel "SERVER> "
   }

}


#
# Open the server for business
#
set server [socket -server serverOpen $Server(port)]

set Server(allDone) 0

# Wait until the allDone flag is set.

vwait Server(allDone)
